/*
  Wrapper for inline functions
 */

#ifdef USE_PRAGMA_IMPLEMENTATION
#pragma implementation        // gcc: Class implementation
#endif

//#define INSIDE_HA_INNOBASE_CC
#define LOCK_MODULE_IMPLEMENTATION

/* Include necessary InnoDB headers */
#include "univ.i"
#include "os0file.h"
#include "os0thread.h"
#include "srv0start.h"
#include "srv0srv.h"
#include "trx0roll.h"
#include "trx0trx.h"
#include "trx0sys.h"
#include "mtr0mtr.h"
#include "row0ins.h"
#include "row0mysql.h"
#include "row0sel.h"
#include "row0upd.h"
#include "log0log.h"
#include "lock0lock.h"
#include "lock0priv.h"
#include "dict0crea.h"
#include "btr0cur.h"
#include "btr0btr.h"
#include "fsp0fsp.h"
#include "sync0sync.h"
#include "fil0fil.h"
#include "trx0xa.h"
#include "thr0loc.h"
#include "ha_prototypes.h"


/*
  inline function from lock0lock.c
 */

/*************************************************************************
Gets the first record lock on a page, where the page is identified by its
file address. */
UNIV_INLINE
lock_t*
lock_rec_get_first_on_page_addr(
/*============================*/
                        /* out: first lock, NULL if none exists */
        ulint   space,  /* in: space */
        ulint   page_no)/* in: page number */
{
        lock_t* lock;

        ut_ad(mutex_own(&kernel_mutex));

        lock = (lock_t*)HASH_GET_FIRST(lock_sys->rec_hash,
                              lock_rec_hash(space, page_no));
        while (lock) {
                if ((lock->un_member.rec_lock.space == space)
                    && (lock->un_member.rec_lock.page_no == page_no)) {

                        break;
                }

                lock = (lock_t*)HASH_GET_NEXT(hash, lock);
        }

        return(lock);
}


/*************************************************************************
Gets the first or next record lock on a page. */
UNIV_INLINE
lock_t*
lock_rec_get_next_on_page(
/*======================*/
                        /* out: next lock, NULL if none exists */
        lock_t* lock)   /* in: a record lock */
{
        ulint   space;
        ulint   page_no;

        ut_ad(mutex_own(&kernel_mutex));
        ut_ad(lock_get_type(lock) == LOCK_REC);

        space = lock->un_member.rec_lock.space;
        page_no = lock->un_member.rec_lock.page_no;

        for (;;) {
                lock = (lock_t*)HASH_GET_NEXT(hash, lock);

                if (!lock) {

                        break;
                }

                if ((lock->un_member.rec_lock.space == space)
                    && (lock->un_member.rec_lock.page_no == page_no)) {

                        break;
                }
        }

        return(lock);
}

/*************************************************************************
Gets the nth bit of a record lock. */
UNIV_INLINE
ibool
lock_rec_get_nth_bit(
/*=================*/
                        /* out: TRUE if bit set */
        lock_t* lock,   /* in: record lock */
        ulint   i)      /* in: index of the bit */
{
        ulint   byte_index;
        ulint   bit_index;
        ulint   b;

        ut_ad(lock);
        ut_ad(lock_get_type(lock) == LOCK_REC);

        if (i >= lock->un_member.rec_lock.n_bits) {

                return(FALSE);
        }

        byte_index = i / 8;
        bit_index = i % 8;

        b = (ulint)*((byte*)lock + sizeof(lock_t) + byte_index);

        return(ut_bit_get_nth(b, bit_index));
}



/*
  Wrapers
 */

/*
  mutex_enter(&kernel_mutex)
 */

void
mutex_enter_wrap(mutex_t* mutex)
{
    mutex_enter_func(mutex, __FILE__, __LINE__);
}

/*
  UT_LIST_GET_FIRST(trx_sys->trx_list)
 */

trx_t*
ut_list_get_first_wrap_trx()
{
    trx_t* trx;

    trx = UT_LIST_GET_FIRST(trx_sys->trx_list);
    return(trx);
}

/*
  lock_rec_find_set_bit(trx->wait_lock)
  not inline
 */

/*
  lock_rec_get_first_on_page_addr(space, page_no)
 */

lock_t*
lock_rec_get_first_on_page_addr_wrap(ulint space, ulint page_no)
{
    return(lock_rec_get_first_on_page_addr(space,page_no));
}

/*
  lock_rec_get_nth_bit(lock, heap_no)
 */

ibool
lock_rec_get_nth_bit_wrap(lock_t* lock, ulint i)
{
    return(lock_rec_get_nth_bit(lock,i));
}


/*
  lock_has_to_wait(trx->wait_lock, lock)
  not inline
 */

/*
  ut_dulint_get_high(trx->id)
 */

ulint
ut_dulint_get_high_wrap(dulint d)
{
    return(ut_dulint_get_high(d));
}


/*
  ut_dulint_get_low(trx->id)
 */

ulint
ut_dulint_get_low_wrap(dulint d)
{
    return(ut_dulint_get_low(d));
}


/*
  lock_rec_get_next_on_page(lock)
 */

lock_t*
lock_rec_get_next_on_page_wrap(lock_t* lock)
{
    return(lock_rec_get_next_on_page(lock));
}


/*
  UT_LIST_GET_FIRST(table->locks);
 */

lock_t*
ut_list_get_first_wrap_locks(dict_table_t* table)
{
    lock_t* lock;

    lock = UT_LIST_GET_FIRST(table->locks);
    return(lock);
}


/*
  UT_LIST_GET_NEXT(un_member.tab_lock.locks, lock);
 */

lock_t*
ut_list_get_next_wrap_locks(lock_t* lock)
{
    lock_t* lock_p;

    lock_p = UT_LIST_GET_NEXT(un_member.tab_lock.locks, lock);
    return(lock_p);
}


/*
  UT_LIST_GET_NEXT(trx_list, trx)
 */

trx_t*
ut_list_get_next_wrap_trx(trx_t* trx)
{
    trx_t* trx_p;

    trx_p = UT_LIST_GET_NEXT(trx_list, trx);
    return(trx_p);
}


/*
  mutex_exit(&kernel_mutex)
 */

void
mutex_exit_wrap(mutex_t* mutex)
{
    mutex_exit(mutex);
}


